/*  Jan 16 2002   */
/* suckmaster2.c  Suckmaster II vacuum robot control program.

 This program is compiled using the Linux shareware C2C compiler 
 available at: http://www.geocities.com/SiliconValley/Network/3656/
 
 The output .asm code is assembled with the Linux freeware assembler
 gpasm avaiable at:  http://gpasm.sourceforge.net 
 
Ratary switch mode select: 
1=Normal (start with vacuum perimeter) 
2=Vacuum random (skip vacuum perimeter)
7=Test GOHOME mode.  (Skip both modes above)

        
-----------------------------------------------------------          
          
 
 */

/* Be sure to set the configuration word to 3f72 when programming the chip
   It's done in the makefile */



#define FOSC   20000000
#pragma  CLOCK_FREQ  20000000

#include "16f877.h"

//#define TMR0DIV   256-(FOSC/(8000*16)) //Compiler will not do this math!
//Divider constant for 8,000 interrupts per second

//For 8000 interrupts/sec
#define TMR0DIV 100



#define BAUD 9600
//#define BAUDDIV (FOSC/(BAUD*64)) - 1
#define BAUDDIV 32



/* these values written into ADCCON0 
   set the A-D channel mux address 
   and converison clock to 1/32 osc.
   This makes a 32uS converson time*/
#define  AD_0           0x81    /* ch 0 Left odometer sensor*/
#define  AD_1           0x89    /* ch 1 Right odometer sensor*/
#define  AD_LMFB        0x99    /* channel 3 Left motor feedback*/
#define  AD_RMFB        0x91    /* channel 2 Right motor feedback*/
#define  AD_4           0xa1     /* ch 4  Drive motor current 1v/amp*/
#define  AD_5           0xa9     /* ch 5      */
#define  AD_6           0xb1     /* ch 6 Battery voltage divided by 10*/
#define  AD_7           0xb9     /* ch 7      */    

char tx_buf[34]@0x20;     /* Tx serial port buffer */

/* Analog inputs */

char  rmfb;
char  lmfb;

//char  mode;


char  right_speed,right_prm;
char  left_speed,left_prm;

char  bot_mode;
char  bump_sw;
char  IR_sens;


#define MODE_STOP 0

#define MODE_VACUUM_PERIMETER 1
#define MODE_RANDOM_VACUUM 2

#define MODE_GOHOME 7
#define MODE_NULL 0xff
#define IGNORE 255

#define REV0 0xf9
#define REV1 0xf6 
#define REV2 0xf2
#define REV3 0xed
#define REV4 0xe7
#define REV5 0xd8
#define FWD0 7
#define FWD1 10
#define FWD2 14
#define FWD3 19
#define FWD4 25
#define FWD5 40

#define HOME_VICINITY 32

//LEDs on PORTD. Clear bit to turn on LED.
#define LF_led 4
#define RF_led 5
#define RS_led 6
#define LS_led 7

char  prescale;
char  timer;
char  prm_ctl;
/* bits in prm_ctl */
#define  R_fwd 0
#define  L_fwd 1


char  workB;

/* Port B motor control bits */

#define  LeftEnable 5    /* Left Motor pwm (L293D enable enable)*/
#define  RightEnable 4   /* Right Motor pwm (L293D enable enable)*/
#define  FwdLeft  3     /*Left Motor forward */
#define  RevLeft  2     /*Left Motor reverse*/
#define  FwdRight 1     /*Right Motor forward */
#define  RevRight 0     /*Right MOtor reverse */


/* These can be tweeked to make the robot track a straight line 
   However, it will not help when the speed is set to max (FWD5)  
   Actually it doesn't work much because of the motor feed back. 
   Putting a gain adj. pot in one of the feedback amps would be better  */

#define  P_width_L   23   /*Left Motor pulse width in interrupt ticks*/
#define  P_width_R   21   /*Right Motor pulse width in interrupt ticks */

char r_speed;
char l_speed;
char l_integrate;
char r_integrate;
char r_speed_set;
char l_speed_set;

char testmode;
char speed_to_pulse_rate(char speed);
char  crcH,crcL;
char tx_ptr,rx_ptr,txstate,rxstate;
char timer2,limit_timer,m_timer,yaw_timer,timerA,escape_timer1,bls_timer,travel_timer;

char home_ctr, zone;
char   runTimer_H;  /* measures run time in seconds */
char   runTimer_L;
char spi_stat;
char  drive_battery;
char  drive_current;

char set_wall_dist;
char flags;
char  r_odometer, l_odometer,spd, speedometer;
char   yaw, nav_yaw;     // +/-18 is +/-90 degrees
char  prev_left_wall_dist, prev_right_wall_dist;
char  delta_left_wall, delta_right_wall, odo_ticks;
char  reversals;

char sonar_disable;
#define SD_RF 0
#define SD_LF 1
#define SD_RS 2
#define SD_LS 3

//These mask cover the left or right beacon bits
//We use them in the trackbeacon function.
#define L_MASK 0x54
#define R_MASK 0xa8


char lf_range,rf_range, rs_range, ls_range, rr_range, lr_range, beacon_range;
char avoidObjectOutput_L, avoidObjectOutput_R;

char trackBeaconOutput_L, trackBeaconOutput_R ;
char vacuumRoomOutput_L, vacuumRoomOutput_R;
char vacuumRandomOutput_L , vacuumRandomOutput_R;
char vacuumPerimeterOutput_L, vacuumPerimeterOutput_R;
char maneuverOutput_L, maneuverOutput_R;
char arb, m_action, m_priority, yaw_limit, ahead_limit;
char wtRight, wtLeft  ;

#define arb_vacuum 1
#define arb_random 2
#define arb_TrackBeacon 3
#define arb_avoid 4
#define arb_escape 5
#define arb_maneuver 6

/* Priorities for maneuver */
#define P_NORMAL 1
#define P_SONAR 2
#define P_BUMP 3
#define P_STALL 4



//Bits in flags variable
#define tick100 0
#define l_odo_last 1
#define r_odo_last 2
#define stalled 3
#define past_center 4
#define unused_bit  5
#define last_seen_right 6
#define last_seen_left 7


char rnd;            /*random number*/
char ctr;           /* a counter */
char temp;
char IR;          /* IR sensor bits */


char vac_rand_state;

//-----------------------------------------------------------
char spi_ptr;         /* SPI buffer pointer */
char spi_buf[9] ;     /* SPI port buffer for sonar data */
//spi_buf offsets...
#define RF_echo 2
#define LF_echo 1
#define RS_echo 3
#define LS_echo 4
#define RR_echo 5
#define LR_echo 6
#define beacon1_dist 7
#define raw_IR 8
//----------------------------------------------------------
//PORTA Right side bump switch, low true.
#define rt_side_bump 4

//PORTC Front bump switch , low true
#define front_bump 2
#define rear_bump 2


//PORTC Rice scooper motor control bit, high true
#define scooper_motor 1




#define COUNT 1      /* offset to count field in tx/rx buffers */
#define ADDRESS 0    /* offset to address field in tx/rx buffers */
#define DATA 2       /* offset to start of data in tx/rx buffers */

#define DIR 0       /* Direction indicator bit 1=right 0=left */



char test_hi@0xfe;

char detect_line();
char neg(char x);
char integrate(char speed,char mfb,char integ);






//------------------------------------------------------------------

/* Interrupts from Timer 0 occur at about 8,000 per second. */
void interrupt(void)
{
   char R_pulse,L_pulse,motor_avg;
   char t1,t2;
   static char divide10;
   static char divide100;
   static char divide25;

   if (PIR1 & 8) {        //if SPI port interrupt...

      set_bit(STATUS,RP0);     //Bank1
      if (SSPSTAT & 1) {       //Check BF buffer full status bit
         clear_bit(STATUS,RP0); //bank 0

         spi_buf[spi_ptr++] = SSPBUF ;

      }
      clear_bit(STATUS,RP0); //bank 0
      clear_bit(PIR1,SSPIF);  //Clear this SPI interrupt
      if (spi_ptr > 8) {        //Got all 9 bytes?
         spi_ptr = 0;
         clear_bit(SSPCON,SSPEN);  //Reset SPI port
         set_bit(SSPCON,SSPEN);
         spi_stat = 0xff;
         lf_range = spi_buf[LF_echo];
         rf_range = spi_buf[RF_echo];
         ls_range = spi_buf[LS_echo];
         rs_range = spi_buf[RS_echo];
         rr_range = spi_buf[RR_echo];
         lr_range = spi_buf[LR_echo];
         if (sonar_disable & (1 << SD_LF)) lf_range  = 255;
         if (sonar_disable & (1 << SD_RF)) rf_range  = 255;
         if (sonar_disable & (1 << SD_LS)) ls_range = 255;
         if (sonar_disable & (1 << SD_RS)) rs_range = 255;
         beacon_range = spi_buf[beacon1_dist];
         IR = spi_buf[raw_IR];

         if (beacon_range < HOME_VICINITY) {   //are we home yet?
            if (home_ctr > 0) home_ctr--;   //5 consecutive "yes" readings required
         } else {                            //..to stop false alarms
            home_ctr = 5;
         }
      }


   }

   if (INTCON & 4) {   //Check for a TMR0 interrupt



      clear_bit(INTCON,2);      //Clear timer 0 interrupt flag
      TMR0 = TMR0DIV + TMR0;       //restart timer 0


      if (prescale-- == 0) {

         prescale = 80;
         if (timer != 0) timer--;    //10 ms pertick timer
         if (timer2 != 0) timer2--;
         if (m_timer != 0) m_timer--;

         /* ramp speed up or down */
         t1 = l_speed + 128;
         t2 = l_speed_set + 128;
         if (t1 < t2 ) l_speed++;
         if (t1 > t2 ) l_speed--;

         t1 = r_speed + 128;
         t2 = r_speed_set + 128;
         if (t1 < t2 ) r_speed++;
         if (t1 > t2 ) r_speed--;



         if (divide10++ >= 10) {
            if (limit_timer != 0) limit_timer--; //100ms per tick timer
            if (yaw_timer != 0) yaw_timer--;
            if (escape_timer1 != 0) escape_timer1--;
            if (bls_timer != 0) bls_timer--;
            if (timerA != 0) timerA--;
            if (travel_timer != 0) travel_timer--;

            divide10 = 0;
            set_bit(flags,tick100);
         }

         if (divide25++ >= 25) {   //update 4 times per sec.
            speedometer = spd;
            spd = 0;
            divide25 = 0;
         }

         if (divide100++ >= 100) {  // 1 second per tick timer
            if (bot_mode != MODE_STOP) {   //Stop run timer when bot stops
               runTimer_L++;
               if (runTimer_L == 0) runTimer_H++;
            }
            if(ls_range == 0) {      //Update wall follow timers
               wtLeft++;
            }else{wtLeft = 0;}

            if(rs_range == 0){
               wtRight++ ;
            }else{wtRight = 0;}


            divide100 = 0;


         }

      }

      PORTB = workB;       //output the prm motor control bits to port B

      workB = 0;


      /* adjust right pulse rate modulator */
      if (right_prm < P_width_R) {
         set_bit(workB,RightEnable);
         if (prm_ctl & (1 << R_fwd))
            set_bit(workB,FwdRight);
         else
            set_bit(workB,RevRight);
      }
      if (right_prm-- == 0) {
         right_prm = speed_to_pulse_rate(right_speed) + P_width_R;
         if (right_speed > 127) clear_bit(prm_ctl,R_fwd);
         else set_bit(prm_ctl,R_fwd);
      }

      /*adjust left pulse rate modulator */
      if (left_prm < P_width_L) {
         set_bit(workB,LeftEnable);
         if (prm_ctl & (1 << L_fwd))
            set_bit(workB,FwdLeft);
         else
            set_bit(workB,RevLeft);
      }

      if (left_prm-- == 0) {
         left_prm = speed_to_pulse_rate(left_speed) + P_width_L;
         if (left_speed > 127) clear_bit(prm_ctl,L_fwd);
         else set_bit(prm_ctl,L_fwd);
      }


   }

   if (bot_mode == 0) workB = 0;    //Stop the bot.

   if ((l_speed == 0) && (r_speed == 0)) workB = 0;
   if (testmode == 1) workB = 0;
   if (drive_current > 45) workB = 0;  // Limit motor current to 900 MA

   //Make a new random number
   if (rnd & 0x80) { //rnd msb is 1
      asm{
         movlw 1ch ;
         xorwf _rnd,F;
         bsf   STATUS,C;
         rlf   _rnd; //8 bit left shift

      }
   } else {    //rnd msb  is 0
      asm{
         bcf   STATUS,C;
         rlf   _rnd; //8 bit left shift

      }
   }

   if (PORTA & 0x20) spi_ptr = 0;  //reset the spi buffer pointer if -SS is high




}
//----------------------------------------------------------------------




//send character if tx ready and return true 
//  else return false if tx not ready 
char tx_byte(char c){
   if (PIR1 & (1 << TXIF)) {

      TXREG = c;
      return 1;     
   } else
      return 0;
}


//-----------------------------------------------------------------------

// compute 16 bit CRC on 1 data byte. 
//   crcH and crcL  are global variables which accumulate the 16 bit CRC. 
//   They must be cleared before starting to compute 16 bit CRC on a string.
//   The final output value will be in crcH and crcL.

void crc(char data){

   char i;
   // short crc16;

   for (i=0;i<8;i++) {                //Do all 8 bits in the data byte
      if (data & 0x80) crcH ^= 0x80; //if data bit is 1 then flip crc msb
      data = data << 1;             // get next data bit
      if (crcH & 0x80) { //crc msb is 1
         asm{
            bcf   STATUS,C;
            rlf   _crcL; //16 bit left shift
            rlf   _crcH;
            movlw 80h;
            xorwf _crcH,F; //xor with 0x8005
            movlw 5;
            xorwf _crcL,F;

         }
      } else {    //crc msb  is 0
         asm{
            bcf   STATUS,C;
            rlf   _crcL; //16 bit left shift
            rlf   _crcH;
         }
      }
   }

}
//-----------------------------------------------------------------------
void set_tx_buffer_crc(){
   int j = tx_buf[COUNT + 0] + 2; //get data size
   int i;

   crcH = 0;         //Clear crc16
   crcL = 0;
   for (i=0;i<j;i++) crc(tx_buf[i]); //Compute crc16 over buffer
   tx_buf[i++] = crcH;      //tack the crc bytes on the end of the buffer
   tx_buf[i++] = crcL;
   txstate = 0;            //State 0 = stopped
}

//-----------------------------------------------------------------------

void make_info_pkt(){
   char i,j;
   i = ADDRESS;
   tx_buf[i] = 1;             // send to master
   i = COUNT;
   tx_buf[i] = 27;            // n bytes
   i = DATA;
   tx_buf[i++] = 0x01;        //data type = 1
   tx_buf[i++] = l_speed;  //1
   tx_buf[i++] = r_speed; //2
   tx_buf[i++] = m_action;  //3
   tx_buf[i++] = runTimer_H;  //4
   tx_buf[i++] = runTimer_L;  //5

   tx_buf[i++] = drive_battery; //6
   tx_buf[i++] = nav_yaw  ;          //7
   tx_buf[i++] = speedometer;   //8 
   tx_buf[i++] = bot_mode;      //9

   tx_buf[i++] = m_priority;          //10
   tx_buf[i++] = 0 ;          //11
   tx_buf[i++] = arb;         //12
   tx_buf[i++] = l_odometer ;   //13  odometer Left
   tx_buf[i++] = r_odometer ;   //14  odometer Right
   tx_buf[i++] = drive_current; //15
   tx_buf[i++] = zone;           //16 IR zone

   for (j=0;j<9;j++) {
      tx_buf[i++] = spi_buf[j];  //16..25 are SPI port buffer data
   }



   crcH = 0;
   crcL = 0;
   set_tx_buffer_crc();  //compute and add CRC16 bits to end of pkt

   txstate = 1;          //Tell scheduler it's ready to transmit
}



//-----------------------------------------------------------------------
void send_tx_buffer(){
   static char j;
   switch (txstate) {
      case 1:
      case 2:
      case 3:  if (tx_byte(0xff)) txstate++;
         break;
      case 4:  if (tx_byte(0xf0)) txstate++;
         break;
      case 5:  if (tx_byte(0x55)) txstate++;
         break;
      case 6:  tx_ptr = 0;
         j = tx_buf[COUNT + 0] + 4; //Must use COUNT+0 to force indirect addressing
         txstate++;
         break;

      case 7:  if (tx_byte(tx_buf[tx_ptr])) {
            j--;
            tx_ptr++ ;
            if (j == 0) txstate = 8;
         }
         break;

      case 8:  if (tx_byte(0x33)) {
            txstate = 0;
         }



   }
}



//-----------------------------------------------------------------------
/* Convert a signed value to an absolute value between 127 and 0 
     0 becomes 127
   127 becomes 0
  -128 becomes 0
  
*/
char speed_to_pulse_rate(char speed)
{
   if (speed > 127) {      //See if speed is negative !
      return speed & 0x7f;
   } else {
      return speed ^ 0x7f;
   }



}

//----------------------------------------------------------------------
char ad_convert(char channel)
{  char x;


   ADCON0 = channel;
   for (x=16;x>0;x--);         //wait at least 20 us for data to stabilize

   set_bit(ADCON0,GO_DONE);   //start A-D conversion

   while (ADCON0 & (1 << NOT_DONE));  //wait for conversion to complete

   return ADRESH+128;          //Return signed result
}

//---------------------------------------------------------------------

void get_AD_values(){

   drive_battery = ad_convert(AD_6) + 128;  //unsigned battery voltage     V/10
   drive_current = ad_convert(AD_7) + 128;  //unsigned drive motor current 1v/amp

}

//-----------------------------------------------------------------------

void adjust_motor_speed(){
   char i;

   rmfb = ad_convert(AD_RMFB); //get right motor feedback
   lmfb = ad_convert(AD_LMFB); //get left motor feedback

   for (i=0;i<10;i++) {  //Adjust motor speeds based on feedback values

      r_integrate = integrate(r_speed,rmfb,r_integrate);
      l_integrate  = integrate(l_speed,lmfb,l_integrate);
      right_speed = r_integrate;
      left_speed  = l_integrate;

   }
}
//----------------------------------------------------------------------
void calc_wall_delta(){

   odo_ticks = 0;
   delta_right_wall = rs_range - prev_right_wall_dist;
   prev_right_wall_dist = rs_range;
   delta_left_wall = ls_range - prev_left_wall_dist;
   prev_left_wall_dist = ls_range;


}

//---------------------------------------------------------------------
/* Read left and right odometer sensors. Also calculates yaw value.
   Speed is computed once per second in the interrupt handler.
   Note yaw value increases(+) with right turns and decreases(-) with left turns*/
void update_odometer(){
   char x;

   x = ad_convert(AD_0) + 128;   //Left odometer sensor
   if ((x > 112) && (~flags & (1 << l_odo_last))) {   //Detect change of state, light to dark
      set_bit(flags,l_odo_last);
      spd++;
      odo_ticks++;
      if (lmfb < 128) {
         if (l_odometer != 127) l_odometer++;    //0.785 inch per tick on odometer
         yaw++;
         nav_yaw++;

      } else {
         if (l_odometer != 128) l_odometer--;
         yaw--; 
         nav_yaw--;

      }
   }




   if ((x < 92) && (flags & (1 << l_odo_last))) {  //Detect change of state,  dark to light
      clear_bit(flags,l_odo_last);
      spd++;
      odo_ticks++;
      if (lmfb < 128) {

         yaw++;           //yaw has twice the resolution of odometer, 0.392 inch per tick
         nav_yaw++;
      } else {

         yaw--;
         nav_yaw--;

      }
   }

   if (odo_ticks >= 16) calc_wall_delta();


   x = ad_convert(AD_1) + 128;    //Right odometer sensor
   if ((x > 112) && (~flags & (1 << r_odo_last))) { //Detect change of state, light to dark
      set_bit(flags,r_odo_last);
      spd++;
      odo_ticks++;
      if (rmfb < 128) {
         if (r_odometer != 127) r_odometer++;
         yaw--;
         nav_yaw--;

      } else {
         if (r_odometer != 128) r_odometer--;
         yaw++;
         nav_yaw++;

      }
   }
   if ((x < 92) && (flags & (1 << r_odo_last))) { //Detect change of state,  dark to light
      clear_bit(flags,r_odo_last);
      spd++;
      odo_ticks++;
      if (rmfb < 128) {
         yaw--;
         nav_yaw--;

      } else {

         yaw++;
         nav_yaw++;

      }

   }

   if (odo_ticks >= 16) calc_wall_delta();


}

//---------------------------------------------------------------------------------

/* This is not used in this program */
/* Saturating addition.  Result is clipped at +127 or -128 
   instead of overflowing.
   Note the parameters a and b and signed char values (-128..+127)
   but the compiler thinks they are unsigned values (0..255) so
   I had to do some strange looking things below.
   */
/*
char addsat(char a, char b)
{  char r;
   if (a > 127) goto a_minus;  //See if a is minus
   if (b > 127) goto a_b_diff; //a is plus, check b for minusness

   r = a + b;                //Both a and b are positive numbers
   if (r > 127) r = 127; //clip at +limit
   return r;

   a_minus:
   if (b < 0x80) goto a_b_diff;  //A is minus, now check b and see if it's minus too
   r = a + b;
   if (r < 0x80) r = 0x80;  //clip at -128
   return r; 

   a_b_diff:
   return a + b;     //No clipping needed


}
*/

//---------------------------------------------------------------------
/* return the absolute value of x */

char abs(char x){
   if (x > 127) {
      x ^= 0xff;
      x++;
   }
   return x;
}


//-----------------------------------------------------------------------
/* return the negative of x */

char neg(char x){
   x ^= 0xff;
   x++;
   return x;
}


//----------------------------------------------------------------------
/* return x if its 0 or less else return 0 */

/*
char clip_pos(char x){
   if (x < 128) x = 0;
   return x;
}

 */
//----------------------------------------------------------------------
/* return x if it's 0 or higher else return 0 */

/*
char clip_neg(char x){
   if (x > 127) x = 0;
   return x;
}

//---------------------------------------------------------------------
//return the a 16 bit value
char sign16(int x){
   char rv;
   rv = 0;
   if (x & 0x8000) rv = 1;
   return rv;
}
//---------------------------------------------------------------------

//Limit x to +/- l
char limit(char x, char l){

   if (x & 0x80) {
      if ( x < neg(l)) return neg(l);
      else return x;
   }
   if ( x > l) return l;
   else return x;


}

*/

//-----------------------------------------------------------

char divideby2(char x){
   if (x < 128) return (x >> 1);
   else {
      return (x >> 1) | 0x80;
   }
}

//------------------------------------------------------------

char get_zone(char IR){
   char z;
   z = 0;
   if (IR & 0x0c) z = 1;
   if (IR & 0x30) z = z + 2 ;
   if (IR & 0xc0) z = z + 4;
   switch (z) {
      case 3:  z = 2; break;
      case 2:  z = 3; break;
      case 4:  z = 5; break;
      case 6:  z = 4; break;
      case 1:  z = 1; break;
      default: z = 0; 
   }
   return z;
}


//----------------------------------------------------------------------
char integrate(char speed,char mfb,char integ)
{
   speed = speed + 128;   //convert to unsigned
   mfb = mfb + 128;
   if (speed > mfb) {
      integ++;
      if (integ == 0x80) integ--;  //Clip at +127
      return integ;
   } else {
      integ--;
      if (integ == 0x7f) integ++;  //Clip at -128
      return integ;
   }



}


//--------------------------------------------------------------------------

//Return true if path (left and right)  is clear in front for "d" inches.
char path_clear(char d )
{
   if(d == IGNORE) return 1;  //If d is 255 always return clear path

   if (d > 7) {
      d = d - 7;   //Subtract the 7 inch dead zone where reading is zero
      d = d << 2;  //mpy by 4 to get sonar range value
   } else {d = 0;}

   if (  (rf_range > d)
         && (lf_range > d)) {
      return 1;} else {return 0;}

}

//------------------------------------------------------------------




//-------------------------------------------------------------------


#define RIGHT 1
#define LEFT 2
#define RandL 3    /* Right and Left */
#define RorL 4     /* Right or Left */

/*
char path_clear_RL(char d, char x)    {
   char rv;
   if(d == IGNORE) return 1;  //If d is 255 always return clear path
   rv = 0;
    if (d > 7) {
       d = d - 7;   //Subtract the 7 inch dead zone where reading is zero
       d = d << 2;  //mpy by 4 to get sonar range value
    } else {d = 0;}
 
    switch(x){
    case RIGHT:   if(rf_range > d) { rv = 1;} break;
    case LEFT:    if(lf_range > d) { rv = 1;} break;
    case RandL:   if((lf_range > d) && (rf_range > d)){ rv = 1;} break;
    case RorL:    if((lf_range > d) || (rf_range > d)){ rf = 1;} break;
    }

    return rv;
 
 }

 */

//---------------------------------------------------------------------


void track_beacon(){
#define sdiff 8

   char b_range[4];
   static char x;
   static char s;
   trackBeaconOutput_L = 0;  
   trackBeaconOutput_R = 0;

   if (bot_mode == MODE_GOHOME) {

      if (x > 1) x = 0;

      if (ls_range == 0) x = 1;
      if (rs_range == 0) x = 0;

      if(limit_timer == 0){
          limit_timer =  13 + (rnd & 3);   //1.3 to 1.6 sec.
          s = s+1;
      }


      if(bls_timer == 0){          //Forget "beacon last seen" after time-out.
          clear_bit(flags,last_seen_left);
          clear_bit(flags,last_seen_right);
      }
     

      if(s & 1){      //Every 1.5 seconds either spin or go straight

         if (x == 1) {
            trackBeaconOutput_L = FWD3;
            trackBeaconOutput_R = REV3;
        
         } else {
            trackBeaconOutput_L = REV3; 
            trackBeaconOutput_R = FWD3;
     
            }
      }else{
            trackBeaconOutput_L = FWD3;
            trackBeaconOutput_R = FWD3;
      }




      if (flags & (1 << last_seen_right)) {
         if (rs_range > 0) {
            trackBeaconOutput_L = FWD3;  //  circle right or...
            trackBeaconOutput_R = FWD1;
            if(beacon_range > 130){
                trackBeaconOutput_L = FWD3; //Spin right if not near evil chair
                trackBeaconOutput_R = REV3; 
            }

         } else {
            trackBeaconOutput_L = FWD3;  //Or go straight if blocked
            trackBeaconOutput_R = FWD3;
         }
      }

      if (flags & (1 << last_seen_left)) {
         if (ls_range > 0) {
            trackBeaconOutput_R = FWD3;  //  circle left.
            trackBeaconOutput_L = FWD1;
             if(beacon_range > 130){        
                 trackBeaconOutput_R = FWD3;   //Spin left if not near the evil chair
                 trackBeaconOutput_L = REV3;
             }                              
         } else {
            trackBeaconOutput_R = FWD3; //Or go straight if blocked on left
            trackBeaconOutput_L = FWD3;
         }
      }






      //if we see the beacon steer towards it.

      if (path_clear(7)) {  //Don't steer towards a reflection from wall!!!

         if (IR & L_MASK) {   //Steer left
            trackBeaconOutput_L = FWD3 - sdiff;
            trackBeaconOutput_R = FWD3 + sdiff;
            set_bit(flags,last_seen_left);    //remember this
            clear_bit(flags,last_seen_right);
            bls_timer = 20;                   //Remember for only 2 secs
         }

         if (IR & R_MASK) { //Steer right
            trackBeaconOutput_L = FWD3 + sdiff;
            trackBeaconOutput_R = FWD3 - sdiff;
            set_bit(flags,last_seen_right);  //remember this
            clear_bit(flags,last_seen_left);
            bls_timer = 20;

         }

         if ((IR & L_MASK) && (IR & R_MASK)) { //Steer straight, we see beacon straight ahead
            trackBeaconOutput_L = FWD4;
            trackBeaconOutput_R = FWD4;
            set_bit(flags,last_seen_right);   //remember this
            set_bit(flags,last_seen_left);
            bls_timer = 20;
           


         }



      }

      //Slow to 1/2 speed when nearing beacon
      if (spi_buf[beacon1_dist] < (HOME_VICINITY + 8) ) {
         trackBeaconOutput_L = divideby2(trackBeaconOutput_L);
         trackBeaconOutput_R = divideby2(trackBeaconOutput_R);
      }

      //Stop when we get really close.

      if (home_ctr == 0) {
         bot_mode = MODE_STOP;
         clear_bit(PORTC,scooper_motor); //Stop rice collector motor
      }



   }else{ s = 1; limit_timer = 15; }    //Initially spin for 1.5 secs


}

//---------------------------------------------------------------------------

/* This is the maneuver state machine.   It can execute a series of simple
   maneuvers.
   
   */

#define A_BACKAWAY 1
#define A_TURN_RIGHT 2
#define A_TURN_LEFT 3
#define A_SPIN_RIGHT 4
#define A_SPIN_LEFT 5
#define A_WALL_LEFT 6
#define A_WALL_RIGHT 7
#define A_AHEAD_SLOW  8
#define A_AHEAD_VERYSLOW 9
#define A_REVERSE_DIR 10
#define A_REVERSE_DIR_B 11
#define A_REVERSE_DIR_2 12
#define A_REVERSE_DIR_B2 13
#define A_WAIT_TIMEOUT 14
#define A_MOVE_FWD 15
#define A_BRAKE_MOTORS 16


#define BRAKE_TIME 5

void maneuver()
{
   static char l_speed_save;
   static char r_speed_save;
   
   

   switch(m_action){

   case A_BACKAWAY:      
         maneuverOutput_L = REV2;
         maneuverOutput_R = REV2;   //Continue backing up if timer not expired

         if(m_timer <= 70){         //Change behavior at the 0.7 sec tick
            m_timer = 200;
            nav_yaw = 0;
            
            if(rs_range > ls_range){
                
               m_action = A_SPIN_RIGHT;   //Spin left
               
            }else{
               m_action = A_SPIN_LEFT;    //Spin left
            }

            if(bot_mode == MODE_VACUUM_PERIMETER){
               if(rs_range > 0){
                  m_action = A_SPIN_RIGHT;
               } else{
                  m_action = A_SPIN_LEFT;
               }
            }

          

         }
         break;


   case A_TURN_RIGHT:
         maneuverOutput_L = FWD4;    //Turn Right                                 
         maneuverOutput_R = FWD0;                                                    
         if (path_clear(ahead_limit)) m_action = 0; //Quit if path clear ahead  
         if (rs_range == 0) m_action = 0;  //Quit turning if we hit something     
         break;

        


   case A_TURN_LEFT:
         maneuverOutput_R = FWD4;    //Turn Left                              
         maneuverOutput_L = FWD0;                                              
         if (path_clear(ahead_limit)) m_action = 0; //Quit if path clear ahead 
         if (ls_range == 0) m_action = 0;  //Quit turning if we hit something  
         
         break;

   case A_SPIN_RIGHT:
          maneuverOutput_L = FWD2;    //Spin Right                               
          maneuverOutput_R = REV2; 
          if((yaw_limit > 0)                //Yaw limit is set
             && (abs(nav_yaw) >= yaw_limit)      // and limit reached and path clear then quit
             && (path_clear(ahead_limit))){
              m_action = A_BRAKE_MOTORS;
              m_timer = BRAKE_TIME;
          }
          if ((path_clear(ahead_limit)) && (yaw_limit == 0)){
              m_action = A_BRAKE_MOTORS; //Quit if path clear ahead 
              m_timer = BRAKE_TIME;
          }

          if (rs_range == 0){
              m_action = 0;  //Quit turning if we hit something;
          }
          break;

   
   case A_BRAKE_MOTORS:
          maneuverOutput_R = 255;   //Speed -1
          maneuverOutput_L = 255;
          break;

   case A_SPIN_LEFT:
         maneuverOutput_R = FWD2;    //Spin Left                              
         maneuverOutput_L = REV2; 
         if((yaw_limit > 0)                //Yaw limit is set
             && (abs(nav_yaw) >= yaw_limit)      // and limit reached and path clear then quit
             && (path_clear(ahead_limit))) {
             m_action = A_BRAKE_MOTORS; //Quit
             m_timer = BRAKE_TIME;
         }

         if ((path_clear(ahead_limit)) && (yaw_limit == 0)){
             m_action = A_BRAKE_MOTORS; //Quit if path clear ahead 
             m_timer = BRAKE_TIME;
         }
         if (ls_range == 0)  m_action = 0;  //Quit turning if we hit something; 
         break;   

           

   case A_WALL_LEFT:
         maneuverOutput_L = FWD0;    //veer right                            
         maneuverOutput_R = REV3;                                           
         if (path_clear(ahead_limit)) m_action = 0; //Quit if path clear ahead 
         break;

   case A_WALL_RIGHT:
         maneuverOutput_R = FWD0;    //veer left                                 
         maneuverOutput_L = REV3;                                                
         if (path_clear(ahead_limit)) m_action = 0; //Quit if path clear ahead 

         break;

   case A_AHEAD_SLOW:
        maneuverOutput_R = FWD3;                                                         
        maneuverOutput_L = FWD3;                                                         
        if (path_clear(ahead_limit)) m_action = 0; //Quit if path clear ahead 
        break;

   case A_AHEAD_VERYSLOW:
          maneuverOutput_R = FWD2;                                                
          maneuverOutput_L = FWD2;                                                
          if (path_clear(ahead_limit)) m_action = 0; //Quit if path clear ahead 
          break;

   case A_REVERSE_DIR:
          l_speed_save = l_speed_set;
          r_speed_save = r_speed_set;
          maneuverOutput_L = neg(l_speed_set);
          maneuverOutput_R = neg(r_speed_set);
          m_action = A_REVERSE_DIR_B;
          break;

   case A_REVERSE_DIR_B:
          maneuverOutput_L = neg(l_speed_save);  
          maneuverOutput_R = neg(r_speed_save);  
          break;

   case A_REVERSE_DIR_2:
         l_speed_save = l_speed_set;
         r_speed_save = r_speed_set;
         maneuverOutput_L = neg(l_speed_set);
         maneuverOutput_R = neg(r_speed_set);
         m_action = A_REVERSE_DIR_B2;
         break;

   case A_REVERSE_DIR_B2:
          maneuverOutput_L = neg(l_speed_save);  
          maneuverOutput_R = neg(r_speed_save);  
          if(m_timer == 0){
             m_timer = 100;
             m_action = A_MOVE_FWD;
          }
          break;





   case A_WAIT_TIMEOUT:
          if(m_timer == 0) m_action = 0;
          break;

   case A_MOVE_FWD:
          maneuverOutput_R = FWD1; 
          maneuverOutput_L = FWD1; 
          if(m_timer == 0) m_action = 0;
          break;

   
   }

   if((m_timer == 0) || (m_action == 0)) {
      m_action = 0;           //Catch all. Limit timer terminates is default.
      m_timer = 0;
      m_priority = 0;
      maneuverOutput_R = 0;
      maneuverOutput_L = 0;
   }



}
//---------------------------------------------------------------------------
void cancel_maneuver()
{
   m_action = 0;
   m_timer = 0;
   maneuverOutput_L = 0;
   maneuverOutput_R = 0;

}

//---------------------------------------------------------------------------
//  Call this function to start a maneuver.  Maneuvers in progress can only
//  be interrupted by a higher priority maneuver.
//  Args: MANEUVER_CODE, time limit 10ms ticks, ahead limit inches, yaw limit, PRIORITY
//  Caller sets the maneuver action and termination conditions in args above.
void start_maneuver(char a, char t, char c, char y, char p)
{
   
   
   if((m_action != 0) && (p > m_priority)){
      cancel_maneuver();
   }


   if(m_action == 0){
      m_action = a;
      m_timer = t;     // 10ms per tick maneuver limit timer (1..255 = 10ms to 2.55 sec. )
      yaw_limit = y;   // set yaw limit if any   (0 = IGNORE)
      ahead_limit = c;  //Set ahead clearence (255=IGNORE)
      m_priority = p;
      
      if(y != 0) nav_yaw = 0;

   }


}

//---------------------------------------------------------------------------




void avoid_object(){

   char x;
   static char action;
#define min_dist 6

   avoidObjectOutput_L = 0;   
   avoidObjectOutput_R = 0;
   if (testmode == 1) return;

   //Set LEDs based on sonar ranges
   if (lf_range < min_dist) clear_bit(PORTD,LF_led);
   if (rf_range < min_dist) clear_bit(PORTD,RF_led);
   if (rs_range < min_dist) clear_bit(PORTD,RS_led);
   if (ls_range < min_dist) clear_bit(PORTD,LS_led);

  if((bot_mode == MODE_GOHOME)
     && (beacon_range < 60)){ return; } //Ignore objects when we are very close to beacon and going home

  
  
  if((runTimer_L < 90) || (bot_mode == MODE_GOHOME)){  //First 90 seconds allow getting close to objects
                                                       //Also when going home to beacon

     if(path_clear(7) == 0){          //7 inches. Must do the shortest distance check first!
        start_maneuver(A_AHEAD_VERYSLOW,10,0,0,P_SONAR);
     }

     if(path_clear(13) == 0){         //13 inches
        start_maneuver(A_AHEAD_SLOW,10,0,0,P_SONAR);
        
     }

  }else {          //After 1.5 minutes start using sonar to go around objects


     //Check shortest ranges first
   
   if (((lf_range < min_dist) && (rf_range < 30))     // 8.5 and 14 inches
       || ((rf_range < min_dist) &&(lf_range < 30))) {  //Object in front of both forward sensors

      if ((rs_range < min_dist) && (ls_range < min_dist)) {
           start_maneuver(A_BACKAWAY,120,12,0,P_SONAR);      // or backup if can't go left or right
      }


      if (lf_range < rf_range) {       //left front less than right front range?
         start_maneuver(A_SPIN_RIGHT,200,12,0,P_SONAR);  //spin Right
      } else {

         if(lf_range == rf_range){    //If both front ranges equal then spin based on side ranges
            if(ls_range > rs_range){
               start_maneuver(A_SPIN_LEFT,200,12,0,P_SONAR);   //spin Left
            }else{
               start_maneuver(A_SPIN_RIGHT,200,12,0,P_SONAR);   //spin Right
            }
         }else{            //Else spin left
            start_maneuver(A_SPIN_LEFT,200,12,0,P_SONAR);   //spin Left
         }
         

      }

      //Then look farther away

      if(path_clear(20) == 0){   //If path not clear ahead for 20 inches...
     
      if(rf_range > lf_range){                //Right front range greater than left front
         start_maneuver(A_TURN_RIGHT,100,16,0,P_SONAR);  // Turn right, 1 sec , 16 inches
       
      }else{
          start_maneuver(A_TURN_LEFT,100,16,0,P_SONAR);  // Turn Left, 1 sec , 16 inches
         
      }
   }




      



   }


  }
  
  

   // If BUMP SWITCH is hit do this:

   if (~PORTC & (1 << front_bump)) {  

      if ((ls_range == 0)
          &&(lf_range > 0)) {          //Hit wall at angle on left side
         start_maneuver(A_WALL_LEFT,50,0,0,P_BUMP);
         
      }

      if ((rs_range == 0)
          && (rf_range > 0)) {          //Hit wall at angle on right side
         start_maneuver(A_WALL_RIGHT,50,0,0,P_BUMP);
        

      }

      if(bot_mode == MODE_VACUUM_PERIMETER) {       //Doing perimeter...
         start_maneuver(A_BACKAWAY,85,8,0,P_BUMP); //Back away then turn until path clear
      }                                                  

      /* Default bump action: 
         Back away, spin left or right 30 degrees 
         then look for clear path ahead 12 inches. */
      start_maneuver(A_BACKAWAY,90,12,5,P_BUMP); 
      


   }



 }



//------------------------------------------------------------------------

// Reverse direction for 1.5 sec if robot is stalled.

void escape(){
   char x;


   x = abs(l_speed) + abs(r_speed);

   if(speedometer != 0) { escape_timer1 = 5; }   // zero speed must persist for at least 0.5 sec to be valid

   if (escape_timer1 == 0){ 
       
      escape_timer1 = 5;
      if((x > 0)
          && (r_speed_set == l_speed_set)) {      //reverse direction if stalled 
         
         start_maneuver(A_REVERSE_DIR,30,0,0,P_STALL);  //Just reverse direction
       }

      if((r_speed_set != l_speed_set)
          && (path_clear(7))){

          start_maneuver(A_REVERSE_DIR_2,50,0,0,P_STALL);   //Spin away from object   

      }


   }  

  }

//-----------------------------------------------------------------------

#define TRK_WALL_R0        1
#define TRK_WALL_SPIN_180  2
#define TRK_WALL_L7        3
#define TRK_WALL_L0_A      4
#define TRK_WALL_L0_B      5

void vacuum_perimeter()
{

   vacuumPerimeterOutput_L = 0;
   vacuumPerimeterOutput_R = 0;
   if(bot_mode != MODE_VACUUM_PERIMETER) return;

   vacuumPerimeterOutput_L = FWD4;    //Hug left wall after startup
   vacuumPerimeterOutput_R = FWD5;
      /*
   if(rs_range > 20){ // 12 in 
       start_maneuver(A_SPIN_RIGHT, 100, IGNORE,15,P_NORMAL); //Spin  right 90 degrees
   }
      */            




}


//--------------------------------------------------------------------------

//vacuum room in random pattern.

#define M_DIST 16
#define VRS_FWD 40
#define VRS_ROTATE_LEFT 41
#define VRS_ROTATE_RIGHT 42
#define VRS_CIRCLE 43


void vacuum_random(){
   static char yaw_limit;
   static char save_state;
   static char r;
   vacuumRandomOutput_L = 0;
   vacuumRandomOutput_R = 0;

   if (bot_mode == MODE_RANDOM_VACUUM) {

      switch (vac_rand_state) {

         case VRS_FWD:  
            save_state = VRS_FWD;
            vacuumRandomOutput_L = FWD5;    //full speed ahead
            vacuumRandomOutput_R = FWD5;

            if (path_clear(20) == 0) {       //20 inches
               vacuumRandomOutput_L = FWD3;  //Slow down if object is near
               vacuumRandomOutput_R = FWD3;
            }

            if (path_clear(12) == 0) {       //12 inches
               vacuumRandomOutput_L = FWD2;  //Slow down more if object is really close
               vacuumRandomOutput_R = FWD2;
            }

            
            if (travel_timer == 0){    //Random rotate left or right every 20 secs
                yaw_limit = 13;      //about 90 degs
                yaw_timer = 10;      //1 sec timeout
                nav_yaw = 0;
                if((rnd & 1) && (ls_range > 0)){
                  vac_rand_state = VRS_ROTATE_LEFT;   //rotate left when time expires
                }
                if(((rnd & 1) == 0) && (rs_range > 0)){ 
                  vac_rand_state = VRS_ROTATE_RIGHT;   //rotate RIGHT when time expires
                }
            }

            if(runTimer_L > 130){  //140 secs 
               r = 1;
               vac_rand_state = VRS_CIRCLE;             //Start circling.
               travel_timer = 200 ; // 20 secs
            }

                    
            break;


      case VRS_CIRCLE:
               save_state = VRS_CIRCLE;
               vacuumRandomOutput_L = FWD4 - r; 
               vacuumRandomOutput_R = FWD4 ;      //Circle left CCW
               if(path_clear(20) == 0){
                    vacuumRandomOutput_L = FWD2;
                    vacuumRandomOutput_R = FWD3;
               }
               if(travel_timer == 0){
                  if (r < 6) r = r + 1;     //Spirial inward
                  travel_timer = 10;
               }

               if(wtLeft > 2){        //turn around 180 degs  if left side against wall
                  start_maneuver(A_SPIN_RIGHT,100,IGNORE,30,P_NORMAL);
               }

              
               break;



         case VRS_ROTATE_LEFT: 
            vacuumRandomOutput_L = REV3;  
            vacuumRandomOutput_R = FWD3;
            set_bit(sonar_disable,SD_RF);
            set_bit(sonar_disable,SD_LF);
            if ((abs(nav_yaw) >= yaw_limit)  //Yaw limit reached
                || (yaw_timer == 0)         //or time out
                || (ls_range == 0)) {         //or object seen on left side
               vac_rand_state = save_state;
               clear_bit(sonar_disable,SD_RF);
               clear_bit(sonar_disable,SD_LF);
               l_odometer = 0;
               r_odometer = 0;
               travel_timer = 200;  //20 secs
            }

            break;



         case VRS_ROTATE_RIGHT:  
            vacuumRandomOutput_L = FWD3;  
            vacuumRandomOutput_R = REV3;
            set_bit(sonar_disable,SD_RF);
            set_bit(sonar_disable,SD_LF);
            if ((abs(nav_yaw) >= yaw_limit)   //Yaw limit reached            
                || (yaw_timer == 0)         //or time out                  
                || (rs_range == 0)) {         //or object seen on right side  
               vac_rand_state = save_state;
               clear_bit(sonar_disable,SD_RF);
               clear_bit(sonar_disable,SD_LF);
               l_odometer = 0;
               r_odometer = 0;
               travel_timer = 200;  //20 secs
            }

            break;

      }





   }






}

//------------------------------------------------------------------------
void arbitrate(){
   char A;

   
     if(vacuumPerimeterOutput_L | vacuumPerimeterOutput_R){
        l_speed_set = vacuumPerimeterOutput_L;
        r_speed_set = vacuumPerimeterOutput_R;
        A = arb_vacuum;
     }
     

   if (vacuumRandomOutput_L | vacuumRandomOutput_R) {
      l_speed_set =  vacuumRandomOutput_L;
      r_speed_set =  vacuumRandomOutput_R;
      A = arb_random;
   }

   if (trackBeaconOutput_L | trackBeaconOutput_R) {
      l_speed_set = trackBeaconOutput_L;
      r_speed_set = trackBeaconOutput_R;
      A = arb_TrackBeacon;
   }
   
   if(maneuverOutput_L | maneuverOutput_R){
      l_speed_set = maneuverOutput_L;                        
      r_speed_set = maneuverOutput_R;                        
      l_speed = l_speed_set;    //Instant speed change required 
      r_speed = r_speed_set;                                    
                                                                
      A = arb_maneuver;
      
   }
   arb = A;

}


//-------------------------------------------------------------------------

void check_battery(){
   if ((drive_battery < 46) && (runTimer_L > 4)) { //After 4 secs and bat less than 9 volts.
      bot_mode = MODE_STOP;
      l_speed = 0;
      r_speed = 0;
      l_speed_set = 0; 
      r_speed_set = 0; 

   }
}

//-------------------------------------------------------------------------


main()
{
   char i;
   char x;
   char y;
   char z;

   //short crc16;

   test_hi = 0;

   INTCON = 0;
   STATUS = 0;
   PORTA  = 0;
   PORTB  = 0;
   PORTC  = 0;
   T1CON  = 0;
   TMR0   = 0;
   workB  = 0x20;
   home_ctr = 4;

   tx_ptr = 0;
   rx_ptr = 0;
   spi_ptr = 0;

   RCSTA = 0x90;             //SPEN + CREN (configure serial port)


   set_bit(STATUS,RP0);     //Bank1


   set_tris_b(0xc0);        //Port B bits are outputs except programmer bits 6,7
   set_tris_c(0xdc);        //Port C outputs except 6:7 are UART pins and 3:4 are SPI pins, bit 2 is front bump sw
   TRISA = 0xff;            //port A all analog inputs
   TRISD = 0x0f;            //port D low nibble inputs, high nibble outputs to LEDs
   TRISE = 7;               //Port E inputs (analog)
   OPTION_REG = 0x01;       //Prescaler 1:4 assigned to Timer 0, pullups on portB
   ADCON1 = 0;              //Set all port A to analog with VREF=VCC
   TXSTA = 0xa2;            //Set tx usart status/ctrl reg= CSRC + TXEN + TRMT
   SPBRG = BAUDDIV;         //Set baudrate
   SSPSTAT = 0x00;          //SPI PORT   slave mode CKE = 1  SMP = 0
   PIE1 = 0;                //Disable all peripheral interrupts
   set_bit(PIE1,SSPIE);     //Enable SPI port interrupts
   clear_bit(STATUS,RP0);   //Bank0

   SSPCON = 0;
   SSPCON = 0x14;            // SPI slave mode -SS enabled clock = SCK pin  CKP=1
   set_bit(SSPCON,SSPEN);    //Enable SPI port

   clear_bit(INTCON,T0IF);  //Clear Timer 0 interrupt flag
   TMR0   = TMR0DIV;        //Start Timer 0
   set_bit(INTCON,T0IE);    //Enable Timer 0 interrupts
   set_bit(INTCON,PEIE);   //Enable peripheral interrupts


   set_bit(INTCON,GIE);     //Enable global interrupts

   clear_bit(PORTC,scooper_motor); //Scooper motor off

   limit_timer = 0;
   m_timer = 0;
   timer2 = 0;
   timerA = 27;   //2.7 sec
   yaw_timer = 0;
   m_action = 0;
   m_priority = 0;
   travel_timer = 250; //25 sec initially before first random turn
   yaw = 0;
   nav_yaw = 0;
   bls_timer = 0;
   escape_timer1 = 21;
   runTimer_H = 0;
   runTimer_L = 0;
   left_speed = 0;
   right_speed = 0;
   r_integrate = 0;
   l_integrate = 0;
   r_speed = 0;
   l_speed = 0;
   r_speed_set = 0;
   l_speed_set = 0;
  
   l_odometer = 0;
   r_odometer = 0;
   wtLeft = 0;
   wtRight = 0;
   sonar_disable = 0;
   speedometer = 11;
   spd = 11;
   testmode = 0;
   clear_bit(flags,past_center);
   beacon_range = 255;
   lf_range = 255;
   rf_range = 255;
   ls_range = 255;
   rs_range = 255;


   set_wall_dist = 0;
   reversals = 0;
   
   vac_rand_state = VRS_FWD;
   bot_mode = MODE_STOP;



   PORTD = 0;              //LEDs on
   timer = 20;             //200 ms
   while (timer > 0);       //Wait for everything to stabilize

   PORTD = 0xf0;  //LEDs off


   make_info_pkt(); //Get new data for transmission
   send_tx_buffer();  //Send data on RF link 


   bot_mode = PORTD & 0xf;    //Initial mode

   if(bot_mode == MODE_GOHOME){     //Are we testing the Go Home mode? ( rotary switch = 7 )
      runTimer_L = 199;             //Yes, advance timer to 199 secs
      bot_mode =  MODE_RANDOM_VACUUM;  //Will switch back to GOHOME in 1 sec.
   }
   
   set_bit(PORTC,scooper_motor); //Start collecting rice



   for (;;) {                //BEGIN MAIN LOOP HERE

      set_bit(PORTC,0);    //Set timing test bit

     

      if(bot_mode == MODE_VACUUM_PERIMETER){

      if(runTimer_L == 12){ bot_mode = MODE_RANDOM_VACUUM; }    //After 12 secs switch modes
    
      }


      if ((runTimer_L >= 200) && (bot_mode != MODE_STOP)) bot_mode = MODE_GOHOME;  // 3:20 minutes:sec
      if (runTimer_L >= 240) bot_mode = MODE_STOP;    // 4 minutes

      z = get_zone(IR) ;
      if (z > 0) zone = z;

      if ((bot_mode == MODE_STOP)  || (testmode == 1)) {clear_bit(PORTC,scooper_motor);} else set_bit(PORTC,scooper_motor);

      if (txstate == 0) {
         make_info_pkt(); //Get new data for transmission
      }

      send_tx_buffer();  //Send data on RF link if TX ready



      PORTD = 0xF0;          //LEDs off

      vacuum_perimeter();
      vacuum_random();
      track_beacon();
      avoid_object();
      escape();
      maneuver();
      arbitrate();
      check_battery();




      clear_bit(PORTC,0);          //Clear timing test bit
      update_odometer();
      get_AD_values();

      if(bot_mode == MODE_STOP) { 
         l_speed_set = 0; 
         r_speed_set = 0;
         r_speed = 0;
         l_speed = 0;
      }
      adjust_motor_speed();

   }   //END MAIN LOOP 




} //End main()
/* End of source */

